/* stat.c -- for Lattice 4.01 or Aztec 5.2b */

#include <exec/types.h>
#include <exec/exec.h>
#include <libraries/dos.h>
#include <libraries/dosextens.h>
#ifdef AZTEC_C
#  include <clib/exec_protos.h>
#  include <clib/dos_protos.h>
#  include <pragmas/exec_lib.h>
#  include <pragmas/dos_lib.h>
#  include "amiga/z-stat.h"		/* fake version of stat.h */
#else
#  include <sys/types.h>
#  include <sys/stat.h>
#  include <proto/exec.h>		/* I think these are only for      */
#  include <proto/dos.h>		/* Lattice versions older than 5.0 */
#endif

/* The Aztec stat.h header is completely unsuitable, since it defines only */
/* a very minimal struct stat with no st_mode field.  So we roll our own,  */
/* which needs to be pulled in by unzip.h for everyone who calls stat().   */

#ifndef SUCCESS
#define SUCCESS (-1)
#define FAILURE (0)
#endif

extern int stat(char *file,struct stat *buf);

stat(file,buf)
char *file;
struct stat *buf;
{

        struct FileInfoBlock *inf;
        struct FileLock *lock;
        long ftime;

        if( (lock = (struct FileLock *)Lock(file,SHARED_LOCK))==0 )
                /* file not found */
                return(-1);

        if( !(inf = (struct FileInfoBlock *)AllocMem(
                (long)sizeof(struct FileInfoBlock),MEMF_PUBLIC|MEMF_CLEAR)) )
        {
                UnLock((BPTR)lock);
                return(-1);
        }

        if( Examine((BPTR)lock,inf)==FAILURE )
        {
                FreeMem((char *)inf,(long)sizeof(*inf));
                UnLock((BPTR)lock);
                return(-1);
        }

        /* fill in buf */

        buf->st_dev                =
        buf->st_nlink        =
        buf->st_uid                =
        buf->st_gid                =
        buf->st_rdev        = 0;
        
        buf->st_ino                = inf->fib_DiskKey;
        buf->st_blocks        = inf->fib_NumBlocks;
        buf->st_size        = inf->fib_Size;
        buf->st_blksize        = 512;

        /* now the date.  AmigaDOG has weird datestamps---
         *      ds_Days is the number of days since 1-1-1978;
         *      however, as Unix wants date since 1-1-1970...
         */

        ftime =
                (inf->fib_Date.ds_Days * 86400 )                +
                (inf->fib_Date.ds_Minute * 60 )                 +
                (inf->fib_Date.ds_Tick / TICKS_PER_SECOND )     +
                (86400 * 8 * 365 )                              +
                (86400 * 2 );  /* two leap years, I think */

/*  ftime += timezone;  */

        buf->st_ctime =
        buf->st_atime =
        buf->st_mtime = ftime;

	buf->st_mode = (inf->fib_DirEntryType < 0 ? S_IFREG : S_IFDIR);

        /* lastly, throw in the protection bits */

        if((inf->fib_Protection & FIBF_READ) == 0)
                buf->st_mode |= S_IREAD;

        if((inf->fib_Protection & FIBF_WRITE) == 0)
                buf->st_mode |= S_IWRITE;

        if((inf->fib_Protection & FIBF_EXECUTE) == 0)
                buf->st_mode |= S_IEXECUTE;

        if((inf->fib_Protection & FIBF_DELETE) == 0)
                buf->st_mode |= S_IDELETE;

        if((inf->fib_Protection & (long)FIBF_ARCHIVE))
                buf->st_mode |= S_IARCHIVE;

        if((inf->fib_Protection & (long)FIBF_PURE))
                buf->st_mode |= S_IPURE;

        if((inf->fib_Protection & (long)FIBF_SCRIPT))
                buf->st_mode |= S_ISCRIPT;

        FreeMem((char *)inf, (long)sizeof(*inf));
        UnLock((BPTR)lock);

        return(0);

}


#ifdef AZTEC_C

/* This here removes unnecessary bulk from the executable with Aztec: */
void _wb_parse()  { }

/* This here pretends we have time zone support and suchlike when we don't: */
long timezone = 0;
void tzset()  { }
/* AmigaDOS only has timezone awareness in release 2.1 and newer. */

int umask()  { return 0; }
/* I think this has something to do with user access privilege? */

#endif

